@@ -3,18 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -37,7 +40,7 @@ IOUSBCommand::NewCommand()
     
     if (me && !me->init())
     {
-	me->free();
+	me->release();
 	me = NULL;
     }
     return me;
@@ -196,6 +199,32 @@ IOUSBCommand::SetReqCount(IOByteCount reqCount)
     _expansionData->_reqCount = reqCount;
 }
 
+void 
+IOUSBCommand::SetRequestMemoryDescriptor(IOMemoryDescriptor *requestMemoryDescriptor) 
+{
+    _expansionData->_requestMemoryDescriptor = requestMemoryDescriptor;
+}
+
+void 
+IOUSBCommand::SetBufferMemoryDescriptor(IOMemoryDescriptor *bufferMemoryDescriptor) 
+{
+    _expansionData->_bufferMemoryDescriptor = bufferMemoryDescriptor;
+}
+
+void
+IOUSBCommand::SetMultiTransferTransaction(bool multiTDTransaction)
+{
+    _expansionData->_multiTransferTransaction = multiTDTransaction;
+}
+
+
+void
+IOUSBCommand::SetFinalTransferInTransaction(bool finalTDinTransaction)
+{
+    _expansionData->_finalTransferInTransaction = finalTDinTransaction;
+}
+
+
 usbCommand 
 IOUSBCommand::GetSelector(void) 
 {
@@ -298,6 +327,29 @@ IOByteCount IOUSBCommand::GetReqCount(void)
     return _expansionData->_reqCount;
 }
 
+IOMemoryDescriptor *IOUSBCommand::GetRequestMemoryDescriptor(void)
+{
+    return _expansionData->_requestMemoryDescriptor;
+}
+
+IOMemoryDescriptor *IOUSBCommand::GetBufferMemoryDescriptor(void)
+{
+    return _expansionData->_bufferMemoryDescriptor;
+}
+
+
+bool IOUSBCommand::GetMultiTransferTransaction(void)
+{
+    return _expansionData->_multiTransferTransaction;
+}
+
+
+bool IOUSBCommand::GetFinalTransferInTransaction(void)
+{
+    return _expansionData->_finalTransferInTransaction;
+}
+
+
 IOUSBIsocCommand*
 IOUSBIsocCommand::NewCommand()
 {
@@ -305,7 +357,7 @@ IOUSBIsocCommand::NewCommand()
     
     if (me && !me->init())
     {
-	me->free();
+	me->release();
 	me = NULL;
     }
     return me;
