@@ -3,18 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -76,6 +79,10 @@ protected:
     struct ExpansionData
     {
         IOByteCount		_reqCount;
+	IOMemoryDescriptor	*_requestMemoryDescriptor;
+	IOMemoryDescriptor	*_bufferMemoryDescriptor;
+	bool			_multiTransferTransaction;
+	bool			_finalTransferInTransaction;
     };
     ExpansionData * 		_expansionData;
     
@@ -110,6 +117,10 @@ public:
     void 			SetCompletionTimeout(UInt32 to);
     void 			SetUIMScratch(UInt32 index, UInt32 value);
     void 			SetReqCount(IOByteCount reqCount);
+    void			SetRequestMemoryDescriptor(IOMemoryDescriptor *requestMemoryDescriptor);
+    void			SetBufferMemoryDescriptor(IOMemoryDescriptor *bufferMemoryDescriptor);
+    void			SetMultiTransferTransaction(bool);
+    void			SetFinalTransferInTransaction(bool);
     usbCommand 			GetSelector(void);
     IOUSBDeviceRequestPtr 	GetRequest(void);
     USBDeviceAddress 		GetAddress(void);
@@ -130,6 +141,10 @@ public:
     UInt32 			GetCompletionTimeout(void);
     UInt32 			GetUIMScratch(UInt32 index);
     IOByteCount 		GetReqCount(void);
+    IOMemoryDescriptor *	GetRequestMemoryDescriptor(void);
+    IOMemoryDescriptor *	GetBufferMemoryDescriptor(void);
+    bool			GetMultiTransferTransaction(void);
+    bool			GetFinalTransferInTransaction(void);
 };
 
 
