@@ -3,18 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -93,10 +96,10 @@ class IOUSBHIDDriver : public IOHIDDevice
     UInt32			_outstandingIO;
     IOCommandGate *		_gate;
     IOUSBPipe *			_interruptOutPipe;
-    UInt32			_maxOutReportSize;
+    UInt32			_maxOutReportSize; 	// Obsolete
     IOBufferMemoryDescriptor *	_outBuffer;
-    UInt32			_deviceUsage;
-    UInt32			_deviceUsagePage;
+    UInt32			_deviceUsage;		// Obsolete
+    UInt32			_deviceUsagePage;	// Obsolete
 
     struct ExpansionData 
     { 
@@ -132,8 +135,6 @@ public:
                         IOMemoryDescriptor ** descriptor ) const;
                         
     virtual OSString * newTransportString() const;
-    virtual OSNumber * newPrimaryUsageNumber() const;
-    virtual OSNumber * newPrimaryUsagePageNumber() const;
 
     virtual OSNumber * newVendorIDNumber() const;
 
