@@ -3,18 +3,21 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.2 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.  
- * Please see the License for the specific language governing rights and 
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
  * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
@@ -38,16 +41,16 @@ BOOL busDevicesChanged=NO;
     [usbextensionsModulesPopup selectItemAtIndex:[prefs integerForKey:@"Kernel Extensions"]];
     [ioregPlanePopup selectItemAtIndex:[prefs integerForKey:@"IORegistry Plane"]];
     [usbLoggerLoggingLevel selectItemAtIndex:[prefs integerForKey:@"Logging Level"]];
-    if ([prefs integerForKey:@"Prober Should Not AutoRefresh"] == 0) {
-        [proberAutoRefreshButton setState:NSOnState];
-        proberShouldAutoRefresh = YES;
-        [proberRefreshButton setEnabled:NO];
-    }
-    else {
+    if ([prefs integerForKey:@"Prober Should AutoRefresh"] == 0) {
         [proberAutoRefreshButton setState:NSOffState];
         proberShouldAutoRefresh = NO;
         [proberRefreshButton setEnabled:YES];
     }
+    else {
+        [proberAutoRefreshButton setState:NSOnState];
+        proberShouldAutoRefresh = YES;
+        [proberRefreshButton setEnabled:NO];
+    }
     if ([prefs integerForKey:@"IORegistry Should Not AutoRefresh"] == 0) {
         [ioregistryAutoRefreshButton setState:NSOnState];
         ioregShouldAutoRefresh = YES;
@@ -70,7 +73,7 @@ BOOL busDevicesChanged=NO;
     [prefs setInteger:[usbextensionsModulesPopup indexOfSelectedItem] forKey:@"Kernel Extensions"];
     [prefs setInteger:[ioregPlanePopup indexOfSelectedItem] forKey:@"IORegistry Plane"];
     [prefs setInteger:[usbLoggerLoggingLevel indexOfSelectedItem] forKey:@"Logging Level"];
-    [prefs setInteger:![proberAutoRefreshButton state] forKey:@"Prober Should Not AutoRefresh"];
+    [prefs setInteger:![proberAutoRefreshButton state] forKey:@"Prober Should AutoRefresh"];
     [prefs setInteger:![ioregistryAutoRefreshButton state] forKey:@"IORegistry Should Not AutoRefresh"];
 
     [prefs synchronize];
@@ -262,6 +265,8 @@ void dumpIter( void *refCon, io_iterator_t iter )
     }
 }
 
+// This method is a drop in replacement for the one that is currently in the file MainController.m
+
 - (IBAction)usbextensionsGetVersions:(id)sender
 {
     NSString *extensionsString = [[NSString alloc] init];
@@ -294,13 +299,9 @@ void dumpIter( void *refCon, io_iterator_t iter )
     outputString = [systemCommands awk:[systemCommands grep:extensionsString arguments:grepArguments] arguments:awkArguments];
     [usbextensionsOutput setString:[NSString stringWithFormat:@"Currently loaded kernel modules:\n\n"]];
     [usbextensionsOutput replaceCharactersInRange:NSMakeRange([[usbextensionsOutput string] length], 0) withString:outputString];
-
-    [extensionsString release];
-    [outputString release];
-    
-    return;
 }
 
+
 - (IBAction)refreshIOReg:(id)sender
 {
     if (sender == ioregPlanePopup) {
